/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_nds32elf_emulation;
extern ld_emulation_xfer_type ld_nds32elf16m_emulation;
extern ld_emulation_xfer_type ld_nds32belf_emulation;
extern ld_emulation_xfer_type ld_nds32belf16m_emulation;

#define EMULATION_LIST \
  &ld_nds32elf_emulation, \
  &ld_nds32elf16m_emulation, \
  &ld_nds32belf_emulation, \
  &ld_nds32belf16m_emulation, \
  0
